Enum LambdaParameter

pub enum LambdaParameter {
    Single((Box<str>, OnionObject)),
    Multiple(Box<[LambdaParameter]>),
}

Variants



Single((Box<str>, OnionObject))



Multiple(Box<[LambdaParameter]>)

Implementations

Source

impl LambdaParameter

pub fn top(key: &str) -> Self

pub fn bottom(key: &str) -> Self

Source

impl LambdaParameter

pub fn len(&self) -> usize

pub fn constraint_at(&self, index: usize) -> Option<&OnionObject>

按照扁平化的方式依照顺序获取参数的约束定义

pub fn key_at(&self, index: usize) -> Option<&str>

pub fn at(&self, index: usize) -> Option<(&str, &OnionObject)>

pub fn to_onion(&self) -> OnionStaticObject

打包成一个对象

pub fn from_onion(obj: &OnionObject) -> Result<LambdaParameter, RuntimeError>

Source

impl LambdaParameter

pub fn upgrade(&self, collected: &mut Vec<GCArc<OnionObjectCell>>)

Source

impl LambdaParameter

Source

impl LambdaParameter

pub fn flatten_keys(&self) -> Box<[Box<str>]>

Recursively flattens the parameter structure to produce a flat list of parameter names.

For a parameter structure like (a, (b, c)), this will return ["a", "b", "c"].

pub fn flatten_constraints(&self) -> Box<[OnionObject]>

Recursively flattens the parameter structure to produce a flat list of constraint objects.

For a parameter structure like (a: Int, (b: String, c: Bool)), this will return a Vec containing the OnionObject for Int, String, and Bool.