pub enum LambdaParameter {
Single((Box<str>, OnionObject)),
Multiple(Box<[LambdaParameter]>),
}Variants
Single((Box<str>, OnionObject))
Multiple(Box<[LambdaParameter]>)
Implementations
Sourceimpl LambdaParameter
impl LambdaParameter
pub fn len(&self) -> usize
Sourcepub fn constraint_at(&self, index: usize) -> Option<&OnionObject>
pub fn constraint_at(&self, index: usize) -> Option<&OnionObject>
按照扁平化的方式依照顺序获取参数的约束定义
pub fn key_at(&self, index: usize) -> Option<&str>
pub fn at(&self, index: usize) -> Option<(&str, &OnionObject)>
Sourcepub fn to_onion(&self) -> OnionStaticObject
pub fn to_onion(&self) -> OnionStaticObject
打包成一个对象
pub fn from_onion(obj: &OnionObject) -> Result<LambdaParameter, RuntimeError>
Sourceimpl LambdaParameter
impl LambdaParameter
pub fn upgrade(&self, collected: &mut Vec<GCArc<OnionObjectCell>>)
Sourceimpl LambdaParameter
impl LambdaParameter
pub fn unpack_arguments( &self, argument: &OnionObject, ) -> Result<Vec<OnionObject>, RuntimeError>
Sourceimpl LambdaParameter
impl LambdaParameter
Sourcepub fn flatten_keys(&self) -> Box<[Box<str>]>
pub fn flatten_keys(&self) -> Box<[Box<str>]>
Recursively flattens the parameter structure to produce a flat list of parameter names.
For a parameter structure like (a, (b, c)), this will return ["a", "b", "c"].
Sourcepub fn flatten_constraints(&self) -> Box<[OnionObject]>
pub fn flatten_constraints(&self) -> Box<[OnionObject]>
Recursively flattens the parameter structure to produce a flat list of constraint objects.
For a parameter structure like (a: Int, (b: String, c: Bool)),
this will return a Vec containing the OnionObject for Int, String, and Bool.